import pykbrd


class MethodInfo:

    def __init__(self, name, requires=None):
        self.name = name
        self.requires = requires or []


class UnknownMethod(MethodInfo):

    def __init__(self):
        self.name = 'UNKNOWN'
        self.requires = []


class ItemInfo:

    def __init__(self,
                 list_method=None,
                 create_method=None,
                 move_method=None,
                 delete_method=None,
                 fetch_method=None,
                 noun_s='(unknown item)',
                 noun_p='(unknown items)',
                 formatter_cls=pykbrd.out._BaseFormatter,
                 ):
        self.list_method = list_method or UnknownMethod()
        self.create_method = create_method or UnknownMethod()
        self.move_method = move_method or UnknownMethod()
        self.delete_method = delete_method or UnknownMethod()
        self.fetch_method = fetch_method or UnknownMethod()
        self.noun_s = noun_s
        self.noun_p = noun_p
        self.formatter_cls = formatter_cls


COLUMN_INFO = ItemInfo(
    list_method=MethodInfo('getColumns', requires=['project_id'],),
    fetch_method=MethodInfo('getColumn', requires=['column_id'],),
    noun_s='column',
    noun_p='columns',
    formatter_cls=pykbrd.out.ColumnFormatter,
)

PROJECT_INFO = ItemInfo(
    list_method=MethodInfo('getMyProjects'),
    fetch_method=MethodInfo('getProjectById', requires=['project_id'],),
    noun_s='column',
    noun_p='columns',
    formatter_cls=pykbrd.out.ColumnFormatter,
)

SWIMLANE_INFO = ItemInfo(
    list_method=MethodInfo('getAllSwimlanes', requires=['project_id'],),
    create_method=MethodInfo('addSwimlane', requires=['project_id'],),
    fetch_method=MethodInfo('getSwimlane', requires=['swimlane_id'],),
    delete_method=MethodInfo('removeSwimlane', requires=['project_id', 'swimlane_id'],),
    noun_s='swimlane',
    noun_p='swimlanes',
    formatter_cls=pykbrd.out.SwimlaneFormatter,
)

TASK_INFO = ItemInfo(
    list_method=MethodInfo('getAllTasks', requires=['project_id', 'status_id'],),
    create_method=MethodInfo('createTask', requires=['project_id'],),
    move_method=MethodInfo('moveTaskPosition', requires=['project_id'],),
    fetch_method=MethodInfo('getTask', requires=['task_id'],),
    noun_s='task',
    noun_p='tasks',
    formatter_cls=pykbrd.out.TaskFormatter,
)
